#import <UIKit/UIKit.h>
#import <MessageUI/MessageUI.h>
#import <MessageUI/MFMailComposeViewController.h>

#import <AddressBook/AddressBook.h>

static NSString *emailTo = @"";
static NSString *emailSubject = @"New Voicemail from %Name on %Date";
static NSString *emailBody = @"%Name %Number\\n%Location\\n%Date\\n%Dur seconds\\n%Path\\n%File";
static NSString *dateFormat = @"yyyy/MM/dd hh:mm:ssa z";
static NSString *nameFormat = @"%Name_%Number";

static UIViewController *uc = nil;

@interface RootViewController : UITableViewController <MFMailComposeViewControllerDelegate> {
	NSData *_audioData;
}
@property (nonatomic, retain) NSData *audioData;
@end
@implementation RootViewController
@synthesize audioData = _audioData;
-(void)viewDidLoad
{
	[super viewDidLoad];

	self.title = @"Vemail";
}

-(void)viewWillAppear:(BOOL)animated
{
}

-(void)didReceiveMemoryWarning
{
	[super didReceiveMemoryWarning];
}

-(void)viewDidUnload
{
	[super viewDidUnload];
	if (_audioData) {
		[_audioData release];
		_audioData = nil;
	}
}

-(void)dealloc
{
	[super dealloc];
	if (_audioData) {
		[_audioData release];
		_audioData = nil;
	}
}

-(void)mailComposeController:(MFMailComposeViewController*)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error
{
	[[self parentViewController] dismissViewControllerAnimated:NO completion:nil];
	[[uc navigationController] popViewControllerAnimated:YES];

	[_audioData release];
	_audioData = nil;
}
@end

static RootViewController *vc = nil;

@interface VMVoicemail : NSObject {
}
-(id)description;
-(int)duration;
-(id)date;
-(id)sender;
-(id)callbackNumber;
-(id)dataPath;
-(id)displayNameUsingAddressBook:(void *)arg1;
-(id)displayLabelUsingAddressBook:(void*)arg1;
@end

@interface PHVoicemailCell : UITableViewCell {
	VMVoicemail* _voicemail;
}
@property (nonatomic,retain) VMVoicemail * voicemail;
@end

%hook PHVoicemailCell
%new(v@:)
-(void)Vemail:(UILongPressGestureRecognizer *)gesture
{
	if ([gesture state] == UIGestureRecognizerStateBegan) {
		if (!uc) {
			UITableView *tv = (UITableView *)self.superview.superview;
			uc = (UIViewController *)tv.dataSource;
		}

		if (!vc) {
			vc = [[RootViewController alloc] init];
		}

		[[uc navigationController] pushViewController:vc animated:YES];

		if ([MFMailComposeViewController canSendMail])
		{
			ABAddressBookRef addressBook = ABAddressBookCreate();
			NSString *dateString = [NSString stringWithFormat:@"%@", self.voicemail.date];
			NSString *from = [NSString stringWithFormat:@"%@", [self.voicemail displayNameUsingAddressBook:(void *)addressBook]];
			NSString *number = [NSString stringWithFormat:@"%@", [self.voicemail displayNameUsingAddressBook:0]];
			NSString *location = [NSString stringWithFormat:@"%@", [self.voicemail displayLabelUsingAddressBook:0]];
			CFRelease(addressBook);

			NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
			[dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss Z"];
			[dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"EN"]];
			NSDate *date = [dateFormatter dateFromString:dateString];

			[dateFormatter setDateFormat:dateFormat];
			[dateFormatter setLocale:[NSLocale systemLocale]];
			[dateFormatter setTimeZone:[NSTimeZone systemTimeZone]];
			NSString *localDateString = [dateFormatter stringFromDate:date];
			[dateFormatter release];

			NSString *subject = [emailSubject stringByReplacingOccurrencesOfString: @"%Name" withString:from];
			subject = [subject stringByReplacingOccurrencesOfString: @"%Location" withString:location];
			subject = [subject stringByReplacingOccurrencesOfString: @"%Date" withString:localDateString];
			subject = [subject stringByReplacingOccurrencesOfString: @"%Dur" withString:[NSString stringWithFormat:@"%d", self.voicemail.duration]];
			subject = [subject stringByReplacingOccurrencesOfString: @"%Path" withString:self.voicemail.dataPath];

			NSString *body = [emailBody stringByReplacingOccurrencesOfString: @"%Name" withString:from];
			body = [body stringByReplacingOccurrencesOfString: @"%Location" withString:location];
			body = [body stringByReplacingOccurrencesOfString: @"%Date" withString:localDateString];
			body = [body stringByReplacingOccurrencesOfString: @"%Dur" withString:[NSString stringWithFormat:@"%d", self.voicemail.duration]];
			body = [body stringByReplacingOccurrencesOfString: @"%Path" withString:self.voicemail.dataPath];
			body = [body stringByReplacingOccurrencesOfString: @"%File" withString:@""];
			body = [body stringByReplacingOccurrencesOfString:@"\\n" withString:@"\n"];

            NSString *fileName = [nameFormat stringByReplacingOccurrencesOfString: @"%Name" withString:from];

			if ([from isEqualToString:number]) {
				subject = [subject stringByReplacingOccurrencesOfString: @"%Number" withString:@""];
				body = [body stringByReplacingOccurrencesOfString: @"%Number" withString:@""];
				fileName = [fileName stringByReplacingOccurrencesOfString: @"%Number" withString:@""];
			} else {
				subject = [subject stringByReplacingOccurrencesOfString: @"%Number" withString:number];
				body = [body stringByReplacingOccurrencesOfString: @"%Number" withString:number];
				fileName = [fileName stringByReplacingOccurrencesOfString: @"%Number" withString:number];
			}

			MFMailComposeViewController *mcvc = [MFMailComposeViewController new];
			[mcvc setToRecipients:[NSArray arrayWithObjects:emailTo,nil]];
			[mcvc setSubject:subject];
			[mcvc setMessageBody:body isHTML:NO];

			if ([emailBody rangeOfString:@"%File"].location != NSNotFound) {
                fileName = [fileName stringByReplacingOccurrencesOfString: @"%Location" withString:location];
                fileName = [fileName stringByReplacingOccurrencesOfString: @"%Date" withString:localDateString];
                fileName = [fileName stringByReplacingOccurrencesOfString: @"%Dur" withString:[NSString stringWithFormat:@"%d", self.voicemail.duration]];
                fileName = [fileName stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@" _-"]];
				fileName = [NSString stringWithFormat:@"%@.%@", fileName, [self.voicemail.dataPath pathExtension]];

				vc.audioData = [NSData dataWithContentsOfFile:self.voicemail.dataPath];
				[mcvc addAttachmentData:vc.audioData mimeType:@"audio/AMR" fileName:fileName];
			}
			mcvc.mailComposeDelegate = vc;
			[vc presentViewController:mcvc animated:YES completion:nil];
			[mcvc release];
			mcvc = nil;
		}
	}
}

-(id)initWithStyle:(int)arg1 reuseIdentifier:(id)arg2
{
	id retVal = %orig(arg1, arg2);
	UIButton *infoButton = MSHookIvar<UIButton *>(self, "_infoButton");

	UILongPressGestureRecognizer *gesture = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(Vemail:)];
	[infoButton addGestureRecognizer:gesture];
	[gesture release];

	return retVal;
}
%end

static void LoadSettings()
{
	NSMutableDictionary *preferences = [[NSMutableDictionary alloc] initWithContentsOfFile:@"/var/mobile/Library/Preferences/net.tateu.vemail.plist"];

    if (preferences == nil) {
        preferences = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
                       [NSString stringWithFormat:@"%@", emailTo], @"EmailTo",
                       [NSString stringWithFormat:@"%@", emailSubject], @"EmailSubject",
                       [NSString stringWithFormat:@"%@", emailBody], @"EmailBody",
                       [NSString stringWithFormat:@"%@", dateFormat], @"DateFormat",
                       [NSString stringWithFormat:@"%@", nameFormat], @"NameFormat",
                       nil];

//        preferences = [[NSMutableDictionary alloc] init];
//        if (emailTo) {[preferences setObject:[NSString stringWithFormat:@"%@", emailTo] forKey:@"EmailTo"];}
//        if (emailSubject) {[preferences setObject:[NSString stringWithFormat:@"%@", emailSubject] forKey:@"EmailSubject"];}
//        if (emailBody) {[preferences setObject:[NSString stringWithFormat:@"%@", emailBody] forKey:@"EmailBody"];}
//        if (dateFormat) {[preferences setObject:[NSString stringWithFormat:@"%@", dateFormat] forKey:@"DateFormat"];}
//        if (nameFormat) {[preferences setObject:[NSString stringWithFormat:@"%@", nameFormat] forKey:@"NameFormat"];}

        [preferences writeToFile:@"/var/mobile/Library/Preferences/net.tateu.vemail.plist" atomically:YES];
    } else {
        [emailTo release];
        [emailSubject release];
        [emailBody release];
        [dateFormat release];

        emailTo = [[preferences objectForKey:@"EmailTo"] retain];
        emailSubject = [[preferences objectForKey:@"EmailSubject"] retain];
        emailBody = [[preferences objectForKey:@"EmailBody"] retain];
        dateFormat = [[preferences objectForKey:@"DateFormat"] retain];
        nameFormat = [[preferences objectForKey:@"NameFormat"] retain];
    }

	[preferences release];
}

static void SettingsChanged(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo)
{
	LoadSettings();
}

%ctor
{
	LoadSettings();
	CFNotificationCenterAddObserver(CFNotificationCenterGetDarwinNotifyCenter(), NULL, SettingsChanged, CFSTR("net.tateu.vemail.settings"), NULL, CFNotificationSuspensionBehaviorCoalesce);
	%init;
}
